﻿init python:
    # preferences.gl_framerate = 60
    def aabb_collision(box_1, box_2):
        if box_1.x < box_2.x + box_2.width:
            return False
        if box_1.x + box_1.width > box_2.x:
            return False
        if box_1.y < box_2.y + box_2.height:
            return False
        if box_1.y + box_1.height > box_2.y:
            return False
        return True



    class animation_class:
        def __init__(self, frames):
            self.frames = frames
            self.current_delta = 0
            self.current_index = 0
        def get_frame(self, delta):
            self.current_delta += delta
            current_frame = self.frames[self.current_index]
            if self.current_delta > current_frame[1]:
                self.current_delta = 0
                self.current_index += 1
                if self.current_index > len(self.frames)-1:
                    self.current_index = 0
            return self.frames[self.current_index][0]

    class animation_handler:
        def __init__(self, animations):
            self.animations = animations
            self.current_animation = None

        def get_frame(self, delta, animation):
            if type(self.animations[animation]) is str:
                return self.animations[animation]
            if not self.current_animation == animation:
                self.current_animation = animation
                return self.animations[animation].get_frame(delta)
            return self.animations[animation].get_frame(delta)
            

    class actor_obj(renpy.Displayable):
        def __init__(self, animations):
            super(renpy.Displayable,self).__init__()
            self.animations = animations
            self.x = 0
            self.y = 0
            self.speed = 60
            self.facing = 0
            self.frame = 0
            self.animation = "down_idle"
            self.last_time = 0
            self.delta_time = 0
        def move(self, move):
            self.animation = move
        def render(self,width,height,st,at):
            if self.last_time == 0:
                self.last_time = time.perf_counter()
            current_time = time.perf_counter()
            delta_time =  round(current_time - self.last_time, 4)
            self.delta_time = delta_time
            if self.animation in ["up_idle", "down_idle", "left_idle", "right_idle"]:
                pass
            elif self.animation == "up":
                self.y -= self.speed*delta_time
            elif self.animation == "down":
                self.y += self.speed*delta_time
            elif self.animation == "right":
                self.x += self.speed*delta_time
            elif self.animation == "left":
                self.x -= self.speed*delta_time
            

            image = self.animations.get_frame(delta_time * 1000, self.animation)
            self.last_time = current_time
            t = Transform(image, xanchor=0.5, yanchor=0.5)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (self.x, self.y))

            renpy.redraw(self, 0)
            return rv

    class topdown_handler:
        def __init__(self):
            self.stat = "start"
            self.player = actor_obj(main_animation_frames)
            self.current_move = None
        def tick(self):
            pass
        def start(self):
            pass
        def end(self):
            pass
        def move(self, move):
            if self.current_move == move:
                self.current_move = None
                self.player.move(move + "_idle")
                return
            self.current_move = move
            self.player.move(move)

screen topdown_screen():
    default g = topdown_handler()
    style_prefix "topdown"
    fixed xysize 320,180:
        at zoom(6, True)
        add g.player align (0.5, 0.5)

    text str(g.player.delta_time) align .1,.0

    fixed xysize 100,100 align 0.0,1.0 offset 150,-150:
        button xysize 100,100:
            yoffset -100 background "#fff5"
            text "w" align .5,.5
            action Function(g.move, "up") keysym ["K_UP", "w"]
        button xysize 100,100:
            xoffset -100 background "#fff5"
            text "a" align .5,.5
            action Function(g.move, "left") keysym ["K_LEFT", "a"]
        button xysize 100,100:
            yoffset 100 background "#fff5"
            text "s" align .5,.5
            action Function(g.move, "down") keysym ["K_DOWN", "s"]
        button xysize 100,100:
            xoffset 100 background "#fff5"
            text "d" align .5,.5
            action Function(g.move, "right") keysym ["K_RIGHT", "d"]

style topdown_text:
    align (.5,.5)
style topdown_frame:
    align (.5,.5)
style topdown_button:
    align (.5,.5)

init:
    image main_down_1:
        Crop((0, 0, 32, 32), "topdown/main.png")
    image main_down_2:
        Crop((32, 0, 32, 32), "topdown/main.png")
    image main_down_3:
        Crop((64, 0, 32, 32), "topdown/main.png")

    image main_left_1:
        Crop((0, 32, 32, 32), "topdown/main.png")
    image main_left_2:
        Crop((32, 32, 32, 32), "topdown/main.png")
    image main_left_3:
        Crop((64, 32, 32, 32), "topdown/main.png")

    image main_up_1:
        Crop((0, 64, 32, 32), "topdown/main.png")
    image main_up_2:
        Crop((32, 64, 32, 32), "topdown/main.png")
    image main_up_3:
        Crop((64, 64, 32, 32), "topdown/main.png")

    image main_right_1:
        Crop((0, 96, 32, 32), "topdown/main.png")
    image main_right_2:
        Crop((32, 96, 32, 32), "topdown/main.png")
    image main_right_3:
        Crop((64, 96, 32, 32), "topdown/main.png")

    default main_animation_up = animation_class(
            [
                ["main_up_1", 150],
                ["main_up_2", 150],
                ["main_up_3", 150],
                ["main_up_2", 150],
            ]
        )
    default main_animation_left = animation_class(
            [
                ["main_left_1", 150],
                ["main_left_2", 150],
                ["main_left_3", 150],
                ["main_left_2", 150],
            ]
        )
    default main_animation_down = animation_class(
            [
                ["main_down_1", 150],
                ["main_down_2", 150],
                ["main_down_3", 150],
                ["main_down_2", 150],
            ]
        )
    default main_animation_right = animation_class(
            [
                ["main_right_1", 150],
                ["main_right_2", 150],
                ["main_right_3", 150],
                ["main_right_2", 150],
            ]
        )
    
    default main_animation_frames = animation_handler(
            {
                "up": main_animation_up,
                "left": main_animation_left,
                "down": main_animation_down,
                "right": main_animation_right,

                "up_idle": "main_up_2",
                "left_idle": "main_left_2",
                "down_idle": "main_down_2",
                "right_idle": "main_right_2",
            }
        )


label topdown_example:
    call screen topdown_screen




